"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var eventemitter3_1 = require("eventemitter3");
// todo: use enum
var CONNECTING = 0;
var OPEN = 1;
var CLOSING = 2;
var CLOSED = 3;
var DEFAULT_RECONNECT_OPTIONS = {
    minDelay: 5 * 1000,
    maxDelay: 60 * 1000,
    growFactor: 1.5,
    maxRetries: Infinity,
};
var EVENTS = {
    OPEN: 'open',
    ERROR: 'error',
    PONG: 'pong',
    MESSAGE: 'message',
    CLOSE: 'close',
};
var DEFAULT_PING_INTERVAL = 60 * 1000;
var WebSocketClient = /** @class */ (function (_super) {
    __extends(WebSocketClient, _super);
    function WebSocketClient(url) {
        var _this = _super.call(this) || this;
        _this.url = url;
        _this.EVENTS = EVENTS; // todo: deprecate
        _this.keepClosed = false;
        _this.pingInterval = DEFAULT_PING_INTERVAL;
        _this.reconnectOptions = DEFAULT_RECONNECT_OPTIONS;
        _this.reconnectTrials = 0;
        _this.reconnectDelay = 0;
        _this.websocketClass = undefined; // 不同环境下的WebSocket来源不同
        _this.on(EVENTS.OPEN, function () {
            _this._maybeResetIdleTimer();
            _this._startPingTimer();
            console.log('on');
            if (_this.reconnectTimer) {
                clearTimeout(_this.reconnectTimer);
                _this.reconnectTimer = undefined;
            }
            _this.reconnectTrials = 0;
            _this.reconnectDelay = 0;
        });
        _this.on(EVENTS.MESSAGE, function (e) {
            _this._maybeResetIdleTimer();
            if (e.data.length === 1) { // pong
                _this.emit(EVENTS.PONG);
            }
            else {
                try {
                    var data = JSON.parse(e.data);
                    var kind = data.kind, payload = data.payload;
                    _this.emit(EVENTS.MESSAGE + ":" + kind, payload);
                }
                catch (e) {
                }
            }
        });
        _this.on(EVENTS.CLOSE, function (e) {
            _this._maybeReconnect();
            _this.pingTimer && clearInterval(_this.pingTimer);
            _this.pingTimer = undefined;
        });
        return _this;
    }
    WebSocketClient.prototype.setReconnectOptions = function (options) {
        this.reconnectOptions = Object.assign({}, DEFAULT_RECONNECT_OPTIONS, options);
    };
    WebSocketClient.prototype._maybeReconnect = function () {
        var _this = this;
        if (this.keepClosed)
            return;
        if (this.sock && this.sock.readyState === 1)
            return;
        if (this.reconnectTrials > this.reconnectOptions.maxRetries)
            return;
        if (this.reconnectTimer) {
            clearTimeout(this.reconnectTimer);
            this.reconnectTimer = undefined;
        }
        var delay = this.reconnectDelay;
        delay = Math.max(delay, this.reconnectOptions.minDelay);
        delay = Math.min(delay, this.reconnectOptions.maxDelay);
        this.reconnectTimer = setTimeout(function () {
            _this.reconnectDelay = delay * _this.reconnectOptions.growFactor;
            _this.open();
        }, delay);
    };
    WebSocketClient.prototype.setPingInterval = function (interval) {
        // 连上之后，如果 <idle> 毫秒内没有消息过来，就自动断开（然后重连）
        this.pingInterval = interval;
    };
    WebSocketClient.prototype.setWebsocketClass = function (websocketClass) {
        this.websocketClass = websocketClass;
    };
    WebSocketClient.prototype._maybeResetIdleTimer = function () {
        var _this = this;
        var maxIdle = this.pingInterval * 2;
        this.idleTimer && clearTimeout(this.idleTimer);
        this.idleTimer = setTimeout(function () {
            _this.idleTimer && _this.close(_this.keepClosed);
        }, maxIdle);
    };
    WebSocketClient.prototype._startPingTimer = function () {
        var _this = this;
        this.pingTimer && clearInterval(this.pingTimer);
        this.pingTimer = setTimeout(function () {
            if (_this.sock && _this.sock.readyState === OPEN) {
                _this.sendPing();
            }
            _this._startPingTimer();
        }, this.pingInterval);
    };
    WebSocketClient.prototype.open = function (url) {
        var _this = this;
        if (url)
            this.url = url;
        if (!this.url)
            throw Error('no url to connect'); // todo: 判断不必要或不全面
        this.sock && this.close(this.keepClosed);
        this.sock = new (this.websocketClass || WebSocket)(this.url);
        this.sock.onopen = function () { return _this.emit(EVENTS.OPEN); };
        this.sock.onerror = function (e) { return _this.emit(EVENTS.ERROR, e); };
        this.sock.onmessage = function (e) { return _this.emit(EVENTS.MESSAGE, e); };
        this.sock.onclose = function (e) { return _this.emit(EVENTS.CLOSE, e); };
    };
    WebSocketClient.prototype.send = function (msg) {
        if (!this.sock)
            throw Error('no WebSocket inited');
        this.sock.send(msg);
    };
    WebSocketClient.prototype.sendPing = function () {
        this.send('.');
    };
    WebSocketClient.prototype.sendMessage = function (kind, payload) {
        this.send(JSON.stringify({ kind: kind, payload: payload }));
    };
    WebSocketClient.prototype.close = function (keepClosed) {
        if (keepClosed === void 0) { keepClosed = true; }
        if (this.sock) {
            if (this.sock.readyState === OPEN)
                this.emit(EVENTS.CLOSE);
            this.keepClosed = keepClosed;
            this.sock.onerror = null;
            this.sock.onmessage = null;
            this.sock.onopen = null;
            this.sock.onclose = null;
            this.sock.close(1000);
            this.sock = undefined;
        }
        this.idleTimer && clearTimeout(this.idleTimer);
    };
    return WebSocketClient;
}(eventemitter3_1.EventEmitter));
exports.default = WebSocketClient;
//# sourceMappingURL=client.js.map